@@ -84,7 +84,7 @@ DRI_CONF_BEGIN
         DRI_CONF_NO_RAST(false)
     DRI_CONF_SECTION_END
     DRI_CONF_SECTION_SOFTWARE
-        DRI_CONF_ARB_VERTEX_PROGRAM(true)
+        DRI_CONF_ARB_VERTEX_PROGRAM(false)
         DRI_CONF_NV_VERTEX_PROGRAM(false)
     DRI_CONF_SECTION_END
 DRI_CONF_END;
@@ -239,9 +239,9 @@ r200CreateScreen( __DRIscreenPrivate *sPriv )
      (PFNGLXSCRENABLEEXTENSIONPROC) (*dri_interface->getProcAddress("glxEnableExtension"));
    void * const psc = sPriv->psc->screenConfigs;
 
-
-   if ( glx_enable_extension == NULL ) {
-      return NULL;
+   if (sPriv->devPrivSize != sizeof(RADEONDRIRec)) {
+      fprintf(stderr,"\nERROR!  sizeof(RADEONDRIRec) does not match passed size from device driver\n");
+      return GL_FALSE;
    }
 
    /* Allocate the private area */
@@ -467,20 +467,21 @@ r200CreateScreen( __DRIscreenPrivate *sPriv )
    screen->driScreen = sPriv;
    screen->sarea_priv_offset = dri_priv->sarea_priv_offset;
 
-   if ( screen->irq != 0 ) {
-      (*glx_enable_extension)( psc, "GLX_SGI_swap_control" );
-      (*glx_enable_extension)( psc, "GLX_SGI_video_sync" );
-      (*glx_enable_extension)( psc, "GLX_MESA_swap_control" );
-   }
+   if ( glx_enable_extension != NULL ) {
+      if ( screen->irq != 0 ) {
+	 (*glx_enable_extension)( psc, "GLX_SGI_swap_control" );
+	 (*glx_enable_extension)( psc, "GLX_SGI_video_sync" );
+	 (*glx_enable_extension)( psc, "GLX_MESA_swap_control" );
+      }
 
-   (*glx_enable_extension)( psc, "GLX_MESA_swap_frame_usage" );
+      (*glx_enable_extension)( psc, "GLX_MESA_swap_frame_usage" );
+      (*glx_enable_extension)( psc, "GLX_MESA_allocate_memory" );
+   }
 
    sPriv->psc->allocateMemory = (void *) r200AllocateMemoryMESA;
    sPriv->psc->freeMemory     = (void *) r200FreeMemoryMESA;
    sPriv->psc->memoryOffset   = (void *) r200GetMemoryOffsetMESA;
 
-   (*glx_enable_extension)( psc, "GLX_MESA_allocate_memory" );
-
    return screen;
 }
 
