@@ -44,7 +44,9 @@
 #define MGA_DEFAULT_USEC_TIMEOUT	10000
 #define MGA_FREELIST_DEBUG		0
 
-static int mga_do_cleanup_dma(drm_device_t * dev);
+#define MINIMAL_CLEANUP    0
+#define FULL_CLEANUP       1
+static int mga_do_cleanup_dma(drm_device_t * dev, int full_cleanup);
 
 /* ================================================================
  * Engine control
@@ -391,7 +393,7 @@ int mga_freelist_put(drm_device_t * dev, drm_buf_t * buf)
  * DMA initialization, cleanup
  */
 
-int mga_driver_preinit(drm_device_t *dev, unsigned long flags)
+int mga_driver_load(drm_device_t *dev, unsigned long flags)
 {
 	drm_mga_private_t * dev_priv;
 
@@ -405,6 +407,14 @@ int mga_driver_preinit(drm_device_t *dev, unsigned long flags)
 	dev_priv->usec_timeout = MGA_DEFAULT_USEC_TIMEOUT;
 	dev_priv->chipset = flags;
 
+	dev_priv->mmio_base = drm_get_resource_start(dev, 1);
+	dev_priv->mmio_size = drm_get_resource_len(dev, 1);
+
+	dev->counters += 3;
+	dev->types[6] = _DRM_STAT_IRQ;
+	dev->types[7] = _DRM_STAT_PRIMARY;
+	dev->types[8] = _DRM_STAT_SECONDARY;
+
 	return 0;
 }
 
@@ -427,7 +437,7 @@ static int mga_do_agp_dma_bootstrap(drm_device_t * dev,
 				    drm_mga_dma_bootstrap_t * dma_bs)
 {
 	drm_mga_private_t * const dev_priv = (drm_mga_private_t *) dev->dev_private;
-	const unsigned int warp_size = mga_warp_microcode_size(dev_priv);
+	unsigned int warp_size = mga_warp_microcode_size(dev_priv);
 	int err;
 	unsigned  offset;
 	const unsigned secondary_size = dma_bs->secondary_bin_count
@@ -437,17 +447,16 @@ static int mga_do_agp_dma_bootstrap(drm_device_t * dev,
 	drm_agp_mode_t mode;
 	drm_agp_info_t info;
 
-	
 	/* Acquire AGP. */
 	err = drm_agp_acquire(dev);
 	if (err) {
-		DRM_ERROR("Unable to acquire AGP\n");
+		DRM_ERROR("Unable to acquire AGP: %d\n", err);
 		return err;
 	}
 
 	err = drm_agp_info(dev, &info);
 	if (err) {
-		DRM_ERROR("Unable to get AGP info\n");
+		DRM_ERROR("Unable to get AGP info: %d\n", err);
 		return err;
 	}
 
@@ -458,7 +467,6 @@ static int mga_do_agp_dma_bootstrap(drm_device_t * dev,
 		return err;
 	}
 
-
 	/* In addition to the usual AGP mode configuration, the G200 AGP cards
 	 * need to have the AGP mode "manually" set.
 	 */
@@ -466,8 +474,7 @@ static int mga_do_agp_dma_bootstrap(drm_device_t * dev,
 	if (dev_priv->chipset == MGA_CARD_TYPE_G200) {
 		if (mode.mode & 0x02) {
 			MGA_WRITE(MGA_AGP_PLL, MGA_AGP2XPLL_ENABLE);
-		}
-		else {
+		} else {
 			MGA_WRITE(MGA_AGP_PLL, MGA_AGP2XPLL_DISABLE);
 		}
 	}
@@ -475,7 +482,7 @@ static int mga_do_agp_dma_bootstrap(drm_device_t * dev,
 
 	/* Allocate and bind AGP memory. */
 	dev_priv->agp_pages = agp_size / PAGE_SIZE;
-	dev_priv->agp_mem = drm_alloc_agp( dev_priv->agp_pages, 0 );
+	dev_priv->agp_mem = drm_alloc_agp( dev, dev_priv->agp_pages, 0 );
 	if (dev_priv->agp_mem == NULL) {
 		dev_priv->agp_pages = 0;
 		DRM_ERROR("Unable to allocate %uMB AGP memory\n",
@@ -485,15 +492,21 @@ static int mga_do_agp_dma_bootstrap(drm_device_t * dev,
 		
 	err = drm_bind_agp( dev_priv->agp_mem, 0 );
 	if (err) {
-		DRM_ERROR("Unable to bind AGP memory\n");
+		DRM_ERROR("Unable to bind AGP memory: %d\n", err);
 		return err;
 	}
 
+	/* Make drm_addbufs happy by not trying to create a mapping for less
+	 * than a page.
+	 */
+	if (warp_size < PAGE_SIZE)
+		warp_size = PAGE_SIZE;
+
 	offset = 0;
 	err = drm_addmap( dev, offset, warp_size,
 			  _DRM_AGP, _DRM_READ_ONLY, & dev_priv->warp );
 	if (err) {
-		DRM_ERROR("Unable to map WARP microcode\n");
+		DRM_ERROR("Unable to map WARP microcode: %d\n", err);
 		return err;
 	}
 
@@ -501,7 +514,7 @@ static int mga_do_agp_dma_bootstrap(drm_device_t * dev,
 	err = drm_addmap( dev, offset, dma_bs->primary_size,
 			  _DRM_AGP, _DRM_READ_ONLY, & dev_priv->primary );
 	if (err) {
-		DRM_ERROR("Unable to map primary DMA region\n");
+		DRM_ERROR("Unable to map primary DMA region: %d\n", err);
 		return err;
 	}
 
@@ -509,7 +522,7 @@ static int mga_do_agp_dma_bootstrap(drm_device_t * dev,
 	err = drm_addmap( dev, offset, secondary_size,
 			  _DRM_AGP, 0, & dev->agp_buffer_map );
 	if (err) {
-		DRM_ERROR("Unable to map secondary DMA region\n");
+		DRM_ERROR("Unable to map secondary DMA region: %d\n", err);
 		return err;
 	}
 
@@ -521,7 +534,7 @@ static int mga_do_agp_dma_bootstrap(drm_device_t * dev,
 
 	err = drm_addbufs_agp( dev, & req );
 	if (err) {
-		DRM_ERROR("Unable to add secondary DMA buffers\n");
+		DRM_ERROR("Unable to add secondary DMA buffers: %d\n", err);
 		return err;
 	}
 
@@ -529,7 +542,7 @@ static int mga_do_agp_dma_bootstrap(drm_device_t * dev,
 	err = drm_addmap( dev, offset, agp_size - offset,
 			  _DRM_AGP, 0, & dev_priv->agp_textures );
 	if (err) {
-		DRM_ERROR("Unable to map AGP texture region\n");
+		DRM_ERROR("Unable to map AGP texture region: %d\n", err);
 		return err;
 	}
 
@@ -552,44 +565,6 @@ static int mga_do_agp_dma_bootstrap(drm_device_t * dev,
 	return 0;
 }
 
-/**
- * Create a "fake" drm_map_t for a pre-mapped range of PCI consistent memory.
- *
- * Unlike \c drm_addmap, this function just creates a \c drm_map_t wrapper for
- * a block of PCI consistent memory.  \c drm_addmap, basically, converts a bus
- * address to a virtual address.  However, \c drm_pci_alloc gives both the bus
- * address and the virtual address for the memory region.  Not only is there
- * no need to map it again, but mapping it again will cause problems.
- * 
- * \param dmah  DRM DMA handle returned by \c drm_pci_alloc.
- * \param map_ptr  Location to store a pointer to the \c drm_map_t.
- * 
- * \returns
- * On success, zero is returned.  Otherwise and error code suitable for
- * returning from an ioctl is returned.
- */
-static int mga_fake_addmap(drm_dma_handle_t * dmah, drm_map_t ** map_ptr)
-{
-	drm_map_t * map;
-
-
-	map = drm_alloc(sizeof(drm_map_t), DRM_MEM_DRIVER);
-	if (map == NULL) {
-		return DRM_ERR(ENOMEM);
-	}
-
-	map->offset = dmah->busaddr;
-	map->size = dmah->size;
-	map->type = _DRM_CONSISTENT;
-	map->flags = _DRM_READ_ONLY;
-	map->handle = dmah->vaddr;
-	map->mtrr = 0;
-	
-	*map_ptr = map;
-	
-	return 0;
-}
-
 /**
  * Bootstrap the driver for PCI DMA.
  * 
@@ -608,7 +583,7 @@ static int mga_do_pci_dma_bootstrap(drm_device_t * dev,
 				    drm_mga_dma_bootstrap_t * dma_bs)
 {
 	drm_mga_private_t * const dev_priv = (drm_mga_private_t *) dev->dev_private;
-	const unsigned int warp_size = mga_warp_microcode_size(dev_priv);
+	unsigned int warp_size = mga_warp_microcode_size(dev_priv);
 	unsigned int primary_size;
 	unsigned int bin_count;
 	int err;
@@ -620,52 +595,48 @@ static int mga_do_pci_dma_bootstrap(drm_device_t * dev,
 		return DRM_ERR(EFAULT);
 	}
 
-
-	/* The WARP microcode base address must be 256-byte aligned.
+	/* Make drm_addbufs happy by not trying to create a mapping for less
+	 * than a page.
 	 */
-	dev_priv->warp_dmah = drm_pci_alloc(dev, warp_size, 0x100, 0x7fffffff);
-	err = mga_fake_addmap(dev_priv->warp_dmah, & dev_priv->warp);
-	if (err) {
-		DRM_ERROR("Unable to map WARP microcode\n");
+	if (warp_size < PAGE_SIZE)
+		warp_size = PAGE_SIZE;
+
+	/* The proper alignment is 0x100 for this mapping */
+	err = drm_addmap(dev, 0, warp_size, _DRM_CONSISTENT,
+			 _DRM_READ_ONLY, &dev_priv->warp);
+	if (err != 0) {
+		DRM_ERROR("Unable to create mapping for WARP microcode: %d\n",
+			  err);
 		return err;
 	}
 
-
 	/* Other than the bottom two bits being used to encode other
 	 * information, there don't appear to be any restrictions on the
 	 * alignment of the primary or secondary DMA buffers.
 	 */
 
-	dev_priv->primary_dmah = NULL;
 	for ( primary_size = dma_bs->primary_size
 	      ; primary_size != 0
 	      ; primary_size >>= 1 ) {
-		dev_priv->primary_dmah = drm_pci_alloc(dev, primary_size,
-					     0x04, 0x7fffffff);
-		if (dev_priv->primary_dmah != NULL) {
+		/* The proper alignment for this mapping is 0x04 */
+		err = drm_addmap(dev, 0, primary_size, _DRM_CONSISTENT,
+				 _DRM_READ_ONLY, &dev_priv->primary);
+		if (!err)
 			break;
-		}
 	}
 
-	if (dev_priv->primary_dmah == NULL) {
-		DRM_ERROR("Unable to allocate primary DMA region\n");
+	if (err != 0) {
+		DRM_ERROR("Unable to allocate primary DMA region: %d\n", err);
 		return DRM_ERR(ENOMEM);
 	}
 
-	if (dev_priv->primary_dmah->size != dma_bs->primary_size) {
+	if (dev_priv->primary->size != dma_bs->primary_size) {
 		DRM_INFO("Primary DMA buffer size reduced from %u to %u.\n",
 			 dma_bs->primary_size, 
-			 (unsigned) dev_priv->primary_dmah->size);
-		dma_bs->primary_size = dev_priv->primary_dmah->size;
+			 (unsigned) dev_priv->primary->size);
+		dma_bs->primary_size = dev_priv->primary->size;
 	}
 
-	err = mga_fake_addmap(dev_priv->primary_dmah, & dev_priv->primary);
-	if (err) {
-		DRM_ERROR("Unable to map primary DMA region\n");
-		return err;
-	}
-
-
 	for ( bin_count = dma_bs->secondary_bin_count
 	      ; bin_count > 0 
 	      ; bin_count-- ) {
@@ -680,7 +651,7 @@ static int mga_do_pci_dma_bootstrap(drm_device_t * dev,
 	}
 	
 	if (bin_count == 0) {
-		DRM_ERROR("Unable to add secondary DMA buffers\n");
+		DRM_ERROR("Unable to add secondary DMA buffers: %d\n", err);
 		return err;
 	}
 
@@ -718,7 +689,7 @@ static int mga_do_dma_bootstrap(drm_device_t * dev,
 	err = drm_addmap( dev, dev_priv->mmio_base, dev_priv->mmio_size,
 			  _DRM_REGISTERS, _DRM_READ_ONLY, & dev_priv->mmio );
 	if (err) {
-		DRM_ERROR("Unable to map MMIO region\n");
+		DRM_ERROR("Unable to map MMIO region: %d\n", err);
 		return err;
 	}
 
@@ -727,7 +698,7 @@ static int mga_do_dma_bootstrap(drm_device_t * dev,
 			  _DRM_READ_ONLY | _DRM_LOCKED | _DRM_KERNEL,
 			  & dev_priv->status );
 	if (err) {
-		DRM_ERROR("Unable to map status region\n");
+		DRM_ERROR("Unable to map status region: %d\n", err);
 		return err;
 	}
 
@@ -747,7 +718,7 @@ static int mga_do_dma_bootstrap(drm_device_t * dev,
 	 */
 
 	if (err) {
-		mga_do_cleanup_dma(dev);
+		mga_do_cleanup_dma(dev, MINIMAL_CLEANUP);
 	}
 
 
@@ -772,6 +743,9 @@ int mga_dma_bootstrap(DRM_IOCTL_ARGS)
 	DRM_DEVICE;
 	drm_mga_dma_bootstrap_t bootstrap;
 	int err;
+	static const int modes[] = { 0, 1, 2, 2, 4, 4, 4, 4 };
+	const drm_mga_private_t * const dev_priv = 
+		(drm_mga_private_t *) dev->dev_private;
 
 
 	DRM_COPY_FROM_USER_IOCTL(bootstrap,
@@ -779,31 +753,25 @@ int mga_dma_bootstrap(DRM_IOCTL_ARGS)
 				 sizeof(bootstrap));
 
 	err = mga_do_dma_bootstrap(dev, & bootstrap);
-	if (! err) {
-		static const int modes[] = { 0, 1, 2, 2, 4, 4, 4, 4 };
-		const drm_mga_private_t * const dev_priv = 
-			(drm_mga_private_t *) dev->dev_private;
-
-		if (dev_priv->agp_textures != NULL) {
-			bootstrap.texture_handle = dev_priv->agp_textures->offset;
-			bootstrap.texture_size = dev_priv->agp_textures->size;
-		}
-		else {
-			bootstrap.texture_handle = 0;
-			bootstrap.texture_size = 0;
-		}
-
-		bootstrap.agp_mode = modes[ bootstrap.agp_mode & 0x07 ];
-		if (DRM_COPY_TO_USER( (void __user *) data, & bootstrap,
-				     sizeof(bootstrap))) {
-			err = DRM_ERR(EFAULT);
-		}
-	}
-	else {
-		mga_do_cleanup_dma(dev);
+	if (err) {
+		mga_do_cleanup_dma(dev, FULL_CLEANUP);
+		return err;
 	}
 
-	return err;
+	if (dev_priv->agp_textures != NULL) {
+		bootstrap.texture_handle = dev_priv->agp_textures->offset;
+		bootstrap.texture_size = dev_priv->agp_textures->size;
+	} else {
+		bootstrap.texture_handle = 0;
+		bootstrap.texture_size = 0;
+	}
+
+	bootstrap.agp_mode = modes[bootstrap.agp_mode & 0x07];
+
+	DRM_COPY_TO_USER_IOCTL((drm_mga_dma_bootstrap_t __user *)data,
+			       bootstrap, sizeof(bootstrap));
+
+	return 0;
 }
 
 
@@ -866,6 +834,7 @@ static int mga_do_init_dma(drm_device_t * dev, drm_mga_init_t * init)
 			DRM_ERROR("failed to find primary dma region!\n");
 			return DRM_ERR(EINVAL);
 		}
+		dev->agp_buffer_token = init->buffers_offset;
 		dev->agp_buffer_map = drm_core_findmap(dev, init->buffers_offset);
 		if (!dev->agp_buffer_map) {
 			DRM_ERROR("failed to find dma buffer region!\n");
@@ -891,14 +860,14 @@ static int mga_do_init_dma(drm_device_t * dev, drm_mga_init_t * init)
 	}
 
 	ret = mga_warp_install_microcode(dev_priv);
-	if (ret < 0) {
-		DRM_ERROR("failed to install WARP ucode!\n");
+	if (ret != 0) {
+		DRM_ERROR("failed to install WARP ucode: %d!\n", ret);
 		return ret;
 	}
 
 	ret = mga_warp_init(dev_priv);
-	if (ret < 0) {
-		DRM_ERROR("failed to init WARP engine!\n");
+	if (ret != 0) {
+		DRM_ERROR("failed to init WARP engine: %d!\n", ret);
 		return ret;
 	}
 
@@ -943,7 +912,7 @@ static int mga_do_init_dma(drm_device_t * dev, drm_mga_init_t * init)
 	return 0;
 }
 
-static int mga_do_cleanup_dma(drm_device_t * dev)
+static int mga_do_cleanup_dma(drm_device_t * dev, int full_cleanup)
 {
 	int err = 0;
 	DRM_DEBUG("\n");
@@ -958,8 +927,8 @@ static int mga_do_cleanup_dma(drm_device_t * dev)
 	if (dev->dev_private) {
 		drm_mga_private_t *dev_priv = dev->dev_private;
 
-		if ((dev_priv->warp != NULL) 
-		    && (dev_priv->mmio->type != _DRM_CONSISTENT))
+		if ((dev_priv->warp != NULL)
+		    && (dev_priv->warp->type != _DRM_CONSISTENT))
 			drm_core_ioremapfree(dev_priv->warp, dev);
 
 		if ((dev_priv->primary != NULL) 
@@ -970,47 +939,8 @@ static int mga_do_cleanup_dma(drm_device_t * dev)
 			drm_core_ioremapfree(dev->agp_buffer_map, dev);
 
 		if (dev_priv->used_new_dma_init) {
-			if (dev_priv->warp != NULL) {
-				drm_rmmap(dev, (void *) dev_priv->warp->offset);
-			}
-
-			if (dev_priv->primary != NULL) {
-				if (dev_priv->primary->type != _DRM_CONSISTENT) {
-					drm_rmmap(dev, (void *) dev_priv->primary->offset);
-				}
-				else {
-					drm_free(dev_priv->primary, sizeof(drm_map_t), DRM_MEM_DRIVER);
-				}
-			}
-
-			if (dev_priv->warp_dmah != NULL) {
-				drm_pci_free(dev, dev_priv->warp_dmah);
-				dev_priv->warp_dmah = NULL;
-			}
-
-			if (dev_priv->primary_dmah != NULL) {
-				drm_pci_free(dev, dev_priv->primary_dmah);
-				dev_priv->primary_dmah = NULL;
-			}
-
-			if (dev_priv->mmio != NULL) {
-				drm_rmmap(dev, (void *) dev_priv->mmio->offset);
-			}
-
-			if (dev_priv->status != NULL) {
-				drm_rmmap(dev, (void *) dev_priv->status->offset);
-			}
-
 			if (dev_priv->agp_mem != NULL) {
-				if (dev->agp_buffer_map != NULL) {
-					drm_rmmap(dev, (void *) dev->agp_buffer_map->offset);
-				}
-
-				if (dev_priv->agp_textures != NULL) {
-					drm_rmmap(dev, (void *) dev_priv->agp_textures->offset);
-					dev_priv->agp_textures = NULL;
-				}
-
+				dev_priv->agp_textures = NULL;
 				drm_unbind_agp(dev_priv->agp_mem);
 
 				drm_free_agp(dev_priv->agp_mem, dev_priv->agp_pages);
@@ -1021,18 +951,20 @@ static int mga_do_cleanup_dma(drm_device_t * dev)
 			if ((dev->agp != NULL) && dev->agp->acquired) {
 				err = drm_agp_release(dev);
 			}
-
-			dev_priv->used_new_dma_init = 0;
 		}
 
 		dev_priv->warp = NULL;
 		dev_priv->primary = NULL;
-		dev_priv->mmio = NULL;
-		dev_priv->status = NULL;
 		dev_priv->sarea = NULL;
 		dev_priv->sarea_priv = NULL;
 		dev->agp_buffer_map = NULL;
 
+		if (full_cleanup) {
+			dev_priv->mmio = NULL;
+			dev_priv->status = NULL;
+			dev_priv->used_new_dma_init = 0;
+		}
+
 		memset(&dev_priv->prim, 0, sizeof(dev_priv->prim));
 		dev_priv->warp_pipe = 0;
 		memset(dev_priv->warp_pipe_phys, 0, sizeof(dev_priv->warp_pipe_phys));
@@ -1060,11 +992,11 @@ int mga_dma_init(DRM_IOCTL_ARGS)
 	case MGA_INIT_DMA:
 		err = mga_do_init_dma(dev, &init);
 		if (err) {
-			(void) mga_do_cleanup_dma(dev);
+			(void) mga_do_cleanup_dma(dev, FULL_CLEANUP);
 		}
 		return err;
 	case MGA_CLEANUP_DMA:
-		return mga_do_cleanup_dma(dev);
+		return mga_do_cleanup_dma(dev, FULL_CLEANUP);
 	}
 
 	return DRM_ERR(EINVAL);
@@ -1193,7 +1125,7 @@ int mga_dma_buffers(DRM_IOCTL_ARGS)
 /**
  * Called just before the module is unloaded.
  */
-int mga_driver_postcleanup(drm_device_t * dev)
+int mga_driver_unload(drm_device_t * dev)
 {
 	drm_free(dev->dev_private, sizeof(drm_mga_private_t), DRM_MEM_DRIVER);
 	dev->dev_private = NULL;
@@ -1204,9 +1136,9 @@ int mga_driver_postcleanup(drm_device_t * dev)
 /**
  * Called when the last opener of the device is closed.
  */
-void mga_driver_pretakedown(drm_device_t * dev)
+void mga_driver_lastclose(drm_device_t * dev)
 {
-	mga_do_cleanup_dma(dev);
+	mga_do_cleanup_dma(dev, FULL_CLEANUP);
 }
 
 int mga_driver_dma_quiescent(drm_device_t * dev)
