@@ -39,12 +39,20 @@
 #define _RADEON_H_
 
 #include "xf86str.h"
+#include "compiler.h"
 
 				/* PCI support */
 #include "xf86Pci.h"
 
-				/* XAA and Cursor Support */
+#ifdef USE_EXA
+#include "exa.h"
+#endif
+#ifdef USE_XAA
 #include "xaa.h"
+#include "xf86fbman.h"
+#endif
+
+				/* Exa and Cursor Support */
 #include "vbe.h"
 #include "xf86Cursor.h"
 
@@ -122,6 +130,7 @@ do {									\
 #define RADEON_ALIGN(x,bytes) (((x) + ((bytes) - 1)) & ~((bytes) - 1))
 #define RADEONPTR(pScrn)      ((RADEONInfoPtr)(pScrn)->driverPrivate)
 
+
 typedef struct {
 				/* Common registers */
     CARD32            ovr_clr;
@@ -291,7 +300,11 @@ typedef enum {
        CHIP_ERRATA_PLL_DELAY           = 0x00000004
 } RADEONErrata;
 
-
+typedef enum {
+	CARD_PCI,
+	CARD_AGP,
+	CARD_PCIE
+} RADEONCardType;
 
 typedef struct {
     CARD32 freq;
@@ -324,6 +337,7 @@ typedef struct {
     CARD32            MemCntl;
     CARD32            BusCntl;
     unsigned long     FbMapSize;        /* Size of frame buffer, in bytes    */
+    unsigned long     FbSecureSize;     /* Size of secured fb area at end of framebuffer */
     int               Flags;            /* Saved copy of mode flags          */
 
 				/* VE/M6 support */
@@ -382,11 +396,25 @@ typedef struct {
 
     Bool              PaletteSavedOnVT; /* Palette saved on last VT switch   */
 
+#ifdef USE_EXA
+    ExaDriverRec      exa;
+#endif
+#ifdef USE_XAA
     XAAInfoRecPtr     accel;
+    int               engineMode;
+#define EXA_ENGINEMODE_UNKNOWN 0
+#define EXA_ENGINEMODE_2D      1
+#define EXA_ENGINEMODE_3D      2
+#endif
     Bool              accelOn;
     xf86CursorInfoPtr cursor;
-    unsigned long     cursor_start;
+#ifdef USE_EXA
+    ExaOffscreenArea   *cursorArea;
+#endif
+    unsigned long     cursor_offset;
+#ifdef USE_XAA
     unsigned long     cursor_end;
+#endif
     Bool              allowColorTiling;
     Bool              tilingEnabled; /* mirror of sarea->tiling_enabled */
 #ifdef ARGB_CURSOR
@@ -395,11 +423,13 @@ typedef struct {
     int               cursor_fg;
     int               cursor_bg;
 
+#ifdef USE_XAA
     /*
      * XAAForceTransBlit is used to change the behavior of the XAA
      * SetupForScreenToScreenCopy function, to make it DGA-friendly.
      */
     Bool              XAAForceTransBlit;
+#endif
 
     int               fifo_slots;       /* Free slots in the FIFO (64 max)   */
     int               pix24bpp;         /* Depth of pixmap for 24bpp fb      */
@@ -416,6 +446,7 @@ typedef struct {
     int               xdir;
     int               ydir;
 
+#ifdef USE_XAA
 				/* ScanlineScreenToScreenColorExpand support */
     unsigned char     *scratch_buffer[1];
     unsigned char     *scratch_save;
@@ -432,7 +463,7 @@ typedef struct {
     int               scanline_hpass;
     int               scanline_x1clip;
     int               scanline_x2clip;
-
+#endif
 				/* Saved values for DashedTwoPointLine */
     int               dashLen;
     CARD32            dashPattern;
@@ -461,7 +492,7 @@ typedef struct {
     drmSize           registerSize;
     drm_handle_t         registerHandle;
 
-    Bool              IsPCI;            /* Current card is a PCI card */
+    RADEONCardType    cardType;            /* Current card is a PCI card */
     drmSize           pciSize;
     drm_handle_t         pciMemHandle;
     unsigned char     *PCI;             /* Map */
@@ -539,6 +570,10 @@ typedef struct {
     int               textureSize;
     int               log2TexGran;
 
+    int               pciGartSize;
+    CARD32            pciGartOffset;
+
+#ifdef USE_XAA
     CARD32            frontPitchOffset;
     CARD32            backPitchOffset;
     CARD32            depthPitchOffset;
@@ -548,6 +583,7 @@ typedef struct {
     FBAreaPtr         backArea;
     int               depthTexLines;
     FBAreaPtr         depthTexArea;
+#endif
 
 				/* Saved scissor values */
     CARD32            sc_left;
@@ -577,7 +613,6 @@ typedef struct {
 				/* XVideo */
     XF86VideoAdaptorPtr adaptor;
     void              (*VideoTimerCallback)(ScrnInfoPtr, Time);
-    FBLinearPtr       videoLinear;
     int               videoKey;
     int		      RageTheatreCrystal;
     int               RageTheatreTunerPort;
@@ -600,19 +635,31 @@ typedef struct {
         CARD8 input[5];
     	} MM_TABLE;
     CARD16 video_decoder_type;
-				/* Render */
+
+    /* Render */
     Bool              RenderAccel;
-    Bool              RenderInited3D;
+#ifdef USE_XAA
     FBLinearPtr       RenderTex;
     void              (*RenderCallback)(ScrnInfoPtr);
     Time              RenderTimeout;
+#endif
 
-				/* general */
+    /* general */
     Bool              showCache;
     OptionInfoPtr     Options;
+
+    Bool              useEXA;
 #ifdef XFree86LOADER
+#ifdef USE_EXA
+    XF86ModReqInfo    exaReq;
+#endif
+#ifdef USE_XAA
     XF86ModReqInfo    xaaReq;
 #endif
+#endif
+
+    /* X itself has the 3D context */
+    Bool              XInited3D;
 
     /* merged fb stuff, also covers clone modes */
     Bool		MergedFB;
@@ -689,11 +736,22 @@ extern void        RADEONWaitForVerticalSync2(ScrnInfoPtr pScrn);
 extern void        RADEONChangeSurfaces(ScrnInfoPtr pScrn);
 
 extern Bool        RADEONAccelInit(ScreenPtr pScreen);
+#ifdef USE_EXA
+extern Bool        RADEONSetupMemEXA (ScreenPtr pScreen);
+extern Bool        RADEONDrawInitMMIO(ScreenPtr pScreen);
+#ifdef XF86DRI
+extern Bool        RADEONDrawInitCP(ScreenPtr pScreen);
+#endif
+#endif
+#ifdef USE_XAA
 extern void        RADEONAccelInitMMIO(ScreenPtr pScreen, XAAInfoRecPtr a);
+#endif
 extern void        RADEONEngineInit(ScrnInfoPtr pScrn);
 extern Bool        RADEONCursorInit(ScreenPtr pScreen);
 extern Bool        RADEONDGAInit(ScreenPtr pScreen);
 
+extern void        RADEONInit3DEngine(ScrnInfoPtr pScrn);
+
 extern int         RADEONMinBits(int val);
 
 extern void        RADEONInitVideo(ScreenPtr pScreen);
@@ -704,11 +762,14 @@ extern void        RADEONPllErrataAfterIndex(RADEONInfoPtr info);
 extern void        RADEONPllErrataAfterData(RADEONInfoPtr info);
 
 #ifdef XF86DRI
+#ifdef USE_XAA
 extern void        RADEONAccelInitCP(ScreenPtr pScreen, XAAInfoRecPtr a);
+#endif
 extern Bool        RADEONDRIScreenInit(ScreenPtr pScreen);
 extern void        RADEONDRICloseScreen(ScreenPtr pScreen);
 extern void        RADEONDRIResume(ScreenPtr pScreen);
 extern Bool        RADEONDRIFinishScreenInit(ScreenPtr pScreen);
+extern void        RADEONDRIAllocatePCIGARTTable(ScreenPtr pScreen);
 
 extern drmBufPtr   RADEONCPGetBuffer(ScrnInfoPtr pScrn);
 extern void        RADEONCPFlushIndirect(ScrnInfoPtr pScrn, int discard);
@@ -719,10 +780,14 @@ extern CARD8*      RADEONHostDataBlit(ScrnInfoPtr pScrn, unsigned int bpp,
 				      unsigned int w, CARD32 dstPitch,
 				      CARD32 *bufPitch, CARD8 **dst,
 				      unsigned int *h, unsigned int *hpass);
-extern void        RADEONHostDataBlitCopyPass(CARD8 *dst, CARD8 *src,
+extern void        RADEONHostDataBlitCopyPass(ScrnInfoPtr pScrn,
+					      unsigned int bpp,
+					      CARD8 *dst, CARD8 *src,
 					      unsigned int hpass,
 					      unsigned int dstPitch,
 					      unsigned int srcPitch);
+extern void        RADEONCopySwap(CARD8 *dst, CARD8 *src, unsigned int size,
+				  int swap);
 
 extern Bool        RADEONGetBIOSInfo(ScrnInfoPtr pScrn, xf86Int10InfoPtr pInt10);
 extern Bool        RADEONGetConnectorInfoFromBIOS (ScrnInfoPtr pScrn);
@@ -903,19 +968,43 @@ do {									\
 #define RADEON_FLUSH_CACHE()						\
 do {									\
     BEGIN_RING(2);							\
-    OUT_RING(CP_PACKET0(RADEON_RB2D_DSTCACHE_CTLSTAT, 0));		\
-    OUT_RING(RADEON_RB2D_DC_FLUSH);					\
+    OUT_RING(CP_PACKET0(RADEON_RB3D_DSTCACHE_CTLSTAT, 0));		\
+    OUT_RING(RADEON_RB3D_DC_FLUSH);					\
     ADVANCE_RING();							\
 } while (0)
 
 #define RADEON_PURGE_CACHE()						\
 do {									\
     BEGIN_RING(2);							\
-    OUT_RING(CP_PACKET0(RADEON_RB2D_DSTCACHE_CTLSTAT, 0));		\
-    OUT_RING(RADEON_RB2D_DC_FLUSH_ALL);					\
+    OUT_RING(CP_PACKET0(RADEON_RB3D_DSTCACHE_CTLSTAT, 0));		\
+    OUT_RING(RADEON_RB3D_DC_FLUSH_ALL);					\
     ADVANCE_RING();							\
 } while (0)
 
 #endif /* XF86DRI */
 
+static __inline__ void RADEON_MARK_SYNC(RADEONInfoPtr info, ScrnInfoPtr pScrn)
+{
+#ifdef USE_EXA
+    if (info->useEXA)
+	exaMarkSync(pScrn->pScreen);
+#endif
+#ifdef USE_XAA
+    if (!info->useEXA)
+	SET_SYNC_FLAG(info->accel);
+#endif
+}
+
+static __inline__ void RADEON_SYNC(RADEONInfoPtr info, ScrnInfoPtr pScrn)
+{
+#ifdef USE_EXA
+    if (info->useEXA)
+	exaWaitSync(pScrn->pScreen);
+#endif
+#ifdef USE_XAA
+    if (!info->useEXA && info->accel)
+	info->accel->Sync(pScrn);
+#endif
+}
+
 #endif /* _RADEON_H_ */
