@@ -2256,9 +2256,9 @@ TDFXScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv) {
 	pTDFX->directRenderingEnabled = TDFXDRIFinishScreenInit(pScreen);
   }
   if (pTDFX->directRenderingEnabled) {
-	xf86DrvMsg(pScrn->scrnIndex, driFrom, "Direct rendering enabled\n");
+	xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Direct rendering enabled\n");
   } else {
-	xf86DrvMsg(pScrn->scrnIndex, driFrom, "Direct rendering disabled\n");
+	xf86DrvMsg(pScrn->scrnIndex, X_WARNING, "Direct rendering disabled\n");
   }
 #endif
 
