@@ -96,6 +96,11 @@
 /* $XConsortium: xf86config.c /main/21 1996/10/28 05:43:57 kaleb $ */
 /* $XdotOrg: xc/programs/Xserver/hw/xfree86/xf86config/xorgconfig.c,v 1.14 2005/05/28 00:08:03 alanc Exp $ */
 
+#ifdef HAVE_CONFIG_H
+# include "xorg-server.h"
+# include "xkb-config.h"
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <ctype.h>
@@ -176,7 +181,7 @@ static int getuid() { return 0; }
 #endif
 
 #ifndef XCONFIGFILE
-#define XCONFIGFILE		"XF86Config"
+#define XCONFIGFILE		"xorg.conf"
 #endif
 #define CONFIGNAME		XCONFIGFILE
 
@@ -256,17 +261,21 @@ static int write_XF86Config(char *filename);
 
 static char *intro_text =
 "\n"
-"This program will create a basic " CONFIGNAME " file, based on menu selections you\n"
-"make.\n"
+"This program will create a basic " CONFIGNAME " file, based on menu selections\n"
+"you make.  It will ask for a pathname when it is ready to write the file.\n"
 "\n"
-"The " CONFIGNAME " file usually resides in " TREEROOTCFG " or /etc/X11. A sample\n"
-CONFIGNAME " file is supplied with "__XSERVERNAME__"; it is configured for a standard\n"
-"VGA card and monitor with 640x480 resolution. This program will ask for a\n"
-"pathname when it is ready to write the file.\n"
+"The " CONFIGNAME " file usually resides in /etc/X11 or " TREEROOTCFG ".  If\n"
+"no " CONFIGNAME " file is present there, " __XSERVERNAME__" will probe the system to\n"
+"autoconfigure itself.  You can run " __XSERVERNAME__ " -configure to generate a " CONFIGNAME "\n"
+"file based on the results of autoconfiguration, or let this program\n"
+"produce a base " CONFIGNAME " file for your configuration, and fine-tune it.\n"
+"A sample " CONFIGNAME " file is also supplied with "__XSERVERNAME__"; it is configured \n"
+"for a standard VGA card and monitor with 640x480 resolution.\n"
 "\n"
-"You can either take the sample " CONFIGNAME " as a base and edit it for your\n"
-"configuration, or let this program produce a base " CONFIGNAME " file for your\n"
-"configuration and fine-tune it.\n"
+"There are also many chipset and card-specific options and settings available,\n"
+"but this program does not know about these. On some configurations some of\n"
+"these settings must be specified. Refer to the X driver man pages and the\n"
+"chipset-specific READMEs in " TREEROOTDOC " for further details.\n"
 #if 0
 " Refer to " TREEROOTDOC "/README.Config\n"
 "for a detailed overview of the configuration process.\n"
@@ -280,19 +289,19 @@ CONFIGNAME " file is supplied with "__XSERVERNAME__"; it is configured for a sta
 "\n"
 "Before continuing with this program, make sure you know what video card\n"
 "you have, and preferably also the chipset it uses and the amount of video\n"
-"memory on your video card. SuperProbe may be able to help with this.\n"
+"memory on your video card, as well as the specifications of your monitor.\n"
 "\n"
 ;
 
 static char *finalcomment_text =
-"File has been written. Take a look at it before running 'startx'. Note that\n"
+"File has been written. Take a look at it before starting an X server. Note that\n"
 "the " CONFIGNAME " file must be in one of the directories searched by the server\n"
 "(e.g. /etc/X11) in order to be used. Within the server press\n"
 "ctrl, alt and '+' simultaneously to cycle video resolutions. Pressing ctrl,\n"
 "alt and backspace simultaneously immediately exits the server (use if\n"
 "the monitor doesn't sync for a particular mode).\n"
 "\n"
-"For further configuration, refer to the " XCONFIGFILE "(5) manual page.\n"
+"For further configuration, refer to the " XCONFIGFILE "(" FILEMANSUFFIX ") manual page.\n"
 "\n";
 
 static void *
@@ -412,7 +421,7 @@ struct {
 	},
 #ifdef sun
 	{"VUID",		&M_VUID,
-	 "Solaris VUID"
+	 "Solaris VUID protocol (SPARC, USB, or virtual mouse)"
 	},
 #endif
 	{"SysMouse",		&M_SYSMOUSE,
@@ -558,7 +567,8 @@ mouse_configuration(void) {
 		emptylines();
 		printf("%s", mouseintro_text);
 		for (j = i; j < i + 14 && j < MOUSETYPE_COUNT; j++)
-			printf("%2d.  %s\n", j + 1, mouse_info[j].name);
+			printf("%2d.  %s [%s]\n", j + 1,
+			       mouse_info[j].name, mouse_info[j].desc);
 		printf("\n");
 		printf("%s", mousecomment_text);
 		printf("Enter a protocol number: ");
@@ -723,6 +733,10 @@ keyboard_configuration(void)
 	}
 #endif
 
+#ifndef XKB_RULES_DIR
+# define XKB_RULES_DIR XKB_BASE_DIRECTORY "/rules"
+#endif
+	
 #ifdef XFREE98_XKB
 	config_xkbrules = "xfree98";	/* static */
         rulesfile = XKB_RULES_DIR "/xfree98";
@@ -2126,25 +2140,33 @@ static char *pointersection_text1 =
 
 static char *pointersection_text2 =
 "\n"
-"# Mouse-speed setting for PS/2 mouse.\n"
-"\n"
-"#    Option \"Resolution\"	\"256\"\n"
-"\n"
 "# When using XQUEUE, comment out the above two lines, and uncomment\n"
 "# the following line.\n"
 "\n"
 "#    Option \"Protocol\"	\"Xqueue\"\n"
 "\n"
+"# Mouse-speed setting for PS/2 mouse.\n"
+"\n"
+"#    Option \"Resolution\"	\"256\"\n"
+"\n"
 "# Baudrate and SampleRate are only for some Logitech mice. In\n"
 "# almost every case these lines should be omitted.\n"
 "\n"
 "#    Option \"BaudRate\"	\"9600\"\n"
 "#    Option \"SampleRate\"	\"150\"\n"
 "\n"
-"# Emulate3Buttons is an option for 2-button Microsoft mice\n"
+"# Mouse wheel mapping.  Default is to map vertical wheel to buttons 4 & 5,\n"
+"# horizontal wheel to buttons 6 & 7.   Change if your mouse has more than\n"
+"# 3 buttons and you need to map the wheel to different button ids to avoid\n"
+"# conflicts.\n"
+"\n"
+"    Option \"ZAxisMapping\"   \"4 5 6 7\"\n"
+"\n"
+"# Emulate3Buttons is an option for 2-button mice\n"
 "# Emulate3Timeout is the timeout in milliseconds (default is 50ms)\n"
 "\n";
 
+
 static char *xinputsection_text =
 "# **********************************************************************\n"
 "# Other input device sections \n"
@@ -2596,8 +2618,9 @@ write_XF86Config(char *filename)
 	 * Write pointer section.
 	 */
 	fprintf(f, "%s", pointersection_text1);
-	fprintf(f, "    Option \"Protocol\"    \"%s\"\n",
-		mouse_info[config_mousetype].name);
+	fprintf(f, "    Option \"Protocol\"    \"%s\"\t# %s\n",
+		mouse_info[config_mousetype].name,
+		mouse_info[config_mousetype].desc);
 #if !defined(__UNIXOS2__) && !defined(QNX4)
 	fprintf(f, "    Option \"Device\"      \"%s\"\n", config_pointerdevice);
 #endif
